﻿using com.superscene.ui;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace parkMonitor.view.util {
	/// <summary>
	/// 按照步骤 1a 或 1b 操作，然后执行步骤 2 以在 XAML 文件中使用此自定义控件。
	///
	/// 步骤 1a) 在当前项目中存在的 XAML 文件中使用该自定义控件。
	/// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
	/// 元素中: 
	///
	///     xmlns:MyNamespace="clr-namespace:parkMonitor.view.treeListBox"
	///
	///
	/// 步骤 1b) 在其他项目中存在的 XAML 文件中使用该自定义控件。
	/// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
	/// 元素中: 
	///
	///     xmlns:MyNamespace="clr-namespace:parkMonitor.view.treeListBox;assembly=parkMonitor.view.treeListBox"
	///
	/// 您还需要添加一个从 XAML 文件所在的项目到此项目的项目引用，
	/// 并重新生成以避免编译错误: 
	///
	///     在解决方案资源管理器中右击目标项目，然后依次单击
	///     “添加引用”->“项目”->[浏览查找并选择此项目]
	///
	///
	/// 步骤 2)
	/// 继续操作并在 XAML 文件中使用控件。
	///
	///     <MyNamespace:TreeListBox/>
	///
	/// </summary>
	public class TreeListBox : TreeView {
		static TreeListBox() {
			DefaultStyleKeyProperty.OverrideMetadata(typeof(TreeListBox), new FrameworkPropertyMetadata(typeof(TreeListBox)));
		}

		public TreeListBox() {
			CmdOnSelectedItem = new SupRelayCommand(new Action(onSelectedItemFun));
		}

		//name列宽
		public static readonly DependencyProperty grdLenNamePro = DependencyProperty.Register("grdLenName", typeof(GridLength), typeof(TreeListBox), new PropertyMetadata(new GridLength(50, GridUnitType.Star)));
		public GridLength grdLenName {
			get { return (GridLength)GetValue(grdLenNamePro); }
			set { SetCurrentValue(grdLenNamePro, value); }
		}

		//value列宽
		public static readonly DependencyProperty grdLenValuePro = DependencyProperty.Register("grdLenValue", typeof(GridLength), typeof(TreeListBox), new PropertyMetadata(new GridLength(50, GridUnitType.Star)));
		public GridLength grdLenValue {
			get { return (GridLength)GetValue(grdLenValuePro); }
			set { SetCurrentValue(grdLenValuePro, value); }
		}

		//选择子项事件
		public static readonly DependencyProperty CmdOnSelectedItemProperty = DependencyProperty.Register("CmdOnSelectedItem", typeof(ICommand), typeof(TreeListBox));
		public ICommand CmdOnSelectedItem {
			get { return (ICommand)GetValue(CmdOnSelectedItemProperty); }
			set { SetCurrentValue(CmdOnSelectedItemProperty, value); }
		}

		private void onSelectedItemFun() {

		}

		public static readonly RoutedEvent OnSelectedItemProperty = EventManager.RegisterRoutedEvent("OnSelectedItem", RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(TreeListBox));
		public event RoutedEventHandler OnSelectedItem {
			//将路由事件添加路由事件处理程序
			add { AddHandler(OnSelectedItemProperty, value); }
			//从路由事件处理程序中移除路由事件
			remove { RemoveHandler(OnSelectedItemProperty, value); }
		}

	}
}
